/* 
BLE Data Glasses
================
Markus Opitz 2024
instructables.com

*/

//BLE
#include <ArduinoBLE.h>
const int characteristicSize = 512;  //max 512
String receivedString;
String command;
//String packet;
char *packet;
BLEService stringService("19B10001-E8F2-537E-4F6C-D104768A1214"); // Create a BLE Service
BLEStringCharacteristic stringCharacteristic("19B10001-E8F2-537E-4F6C-D104768A1214", BLERead | BLEWrite, 512); // Create a BLE Characteristic

//OLED
#include <Arduino.h>
#include <U8g2lib.h>
#include <Wire.h>
#define U8G2_16BIT
u8g2_uint_t offset;	  // offset for the scrolling text
u8g2_uint_t width;		// pixel width of  scrolling text (< 128 unless U8G2_16BIT is defined)

//for XIAO nRF52820 sense:
U8G2_SSD1306_64X32_1F_F_HW_I2C u8g2(U8G2_MIRROR, /* reset=*/ U8X8_PIN_NONE); //U8G2_R0  = no rotation, U8G2_R2  = rotation 180°, U8G2_MIRROR
//for XIAO nRF52820:
//U8G2_SSD1306_128X64_NONAME_F_SW_I2C u8g2(U8G2_R0, /* clock=*/ SCL, /* data=*/ SDA, /* reset=*/ U8X8_PIN_NONE);



void setup() {
  Serial.begin(9600); // Start the Serial communication
  while (!Serial); // Wait for Serial to be ready

//OLED
  u8g2.begin();
  //prepare
  u8g2.setFont(u8g2_font_6x10_tf);
  u8g2.setFontRefHeightExtendedText();
  u8g2.setDrawColor(1);
  u8g2.setFontPosTop();
  u8g2.setFontDirection(0);
  u8g2.setContrast(255);
  u8g2.setFontMode(0);		// enable transparent mode == faster

  makeDot(); //make a dot to see if it is working

  // Initialize BLE
  if (!BLE.begin()) {
    Serial.println("Starting BLE failed!");
    while (1);
  }

  // Set the local name and advertising
  BLE.setLocalName("BLE_Data_Glasses");
  BLE.setAdvertisedService(stringService);

  // Add the characteristic to the service
  stringService.addCharacteristic(stringCharacteristic);
  BLE.addService(stringService);

  // Start advertising
  BLE.advertise();
  Serial.println("BLE String Receiver is now advertising...");
  delay(1000);
}

/*
void loop() {
  crosshair();
}

*/

void loop() {
  // Listen for BLE events
  BLE.poll();

  // Check if a central device has connected
  BLEDevice central = BLE.central();

  if (central) {
    Serial.print("Connected to central: ");
    Serial.println(central.address());

    // Loop while the central is connected
    while (central.connected()) {
      // Check if the characteristic has been written to
      if (stringCharacteristic.written()) {
        receivedString = stringCharacteristic.value();
        Serial.print("Received String: ");
        Serial.println(receivedString);
        command = receivedString;
        //packet = command;
        String firstFour = command.substring(0,4); //pick the first 4 letters/signal word
        

        if (command == "cross") {
              u8g2.clearBuffer();u8g2.clearDisplay();
              drawCrosshair();
              u8g2.sendBuffer();
        }
        if (command == "horizon"){
              u8g2.clearBuffer();u8g2.clearDisplay();
              drawHorizon();
              u8g2.sendBuffer();
        }
        if (firstFour == "time"){
              u8g2.clearBuffer();u8g2.clearDisplay();
              //packet = command;
              command.remove(0, 5); //remove signal word from string
              Serial.println(packet);
              u8g2.setCursor(20, 14);
              u8g2.println(command);
              //u8g2.println(packet);
              //command.toCharArray(packet, 100);
              u8g2.sendBuffer();
              packet = "";
        }
        if (command == "PhoneCallOn"){
            u8g2.clearBuffer();u8g2.clearDisplay();
            drawAlarm();
            u8g2.sendBuffer();  
        }
        if (command == "PhoneCallOff"){
            u8g2.clearBuffer();u8g2.clearDisplay();       
        }
       //check for signal word "text" in the string

        if (firstFour == "text") {
         Serial.println(firstFour);
         Serial.println("switch to teleprompter:");
         command.remove(0, 5); //remove signal word
         Serial.println(packet);
         width = u8g2.getUTF8Width(packet);		// calculate pixel width of text
         scrollText();
         packet = "";
         }
      }
    }

    // When the central disconnects
    Serial.print("Disconnected from central: ");
    Serial.println(central.address());
  }
}


void scrollText() {
  
  command.toCharArray(packet, 100);

  u8g2_uint_t x;
  
  u8g2.firstPage();
  do {
  
    // draw the scrolling text at current offset
    x = offset;
    //u8g2.setFont(u8g2_font_inb30_mr);		// set the target font
    u8g2.setFont(u8g2_font_6x10_tf);
    do {								// repeated drawing of the scrolling text...
      u8g2.drawUTF8(x, 20, packet);			// draw the scolling text
      x += width*4;						// add the pixel width of the scrolling text
    } while( x < u8g2.getDisplayWidth() );		// draw again until the complete display is filled
    
    //u8g2.setFont(u8g2_font_inb16_mr);		// draw the current pixel width
    u8g2.setFont(u8g2_font_6x10_tf);
    u8g2.setCursor(0, 58);
    //u8g2.print(width);					// this value must be lesser than 128 unless U8G2_16BIT is set
    
  } while ( u8g2.nextPage() );
  
  offset-=18;							// scroll by one pixel  == speed
  if ( (u8g2_uint_t)offset < (u8g2_uint_t)-width )	
    offset = 0;							// start over again
    
  delay(200);							// make some delay
}

void crosshair() {
  Serial.println("show crosshair");
  drawCrosshair();
  for (int i=1; i<11; i++) {
    Serial.print(i);
    delay(300);
  }
  Serial.println("");
  delay(500);
}

void drawHorizon() {
  Serial.println(myIMU.readFloatGyroX(), 2);
  Serial.println(myIMU.readFloatGyroZ(), 2);
  int Xvalue = myIMU.readFloatGyroX()/2;
  int Zvalue = myIMU.readFloatGyroZ();
  u8g2.drawLine(0, 16+Xvalue +Zvalue, 64, 16-Xvalue +Zvalue);
}

void time() {
  Serial.println("show time");
  for (int i=1; i<11; i++) {
    Serial.print(i);
    delay(300);
  }
  Serial.println("");
  delay(500);
}

void PhoneCallOn() {
  Serial.println("draw phone call icon");
}
void PhoneCallOff() {
  Serial.println("remove phone call icon");
}


// all OLED subroutines -  -  -  -  -  -  -  -  -  -  -  -

void drawTestPic(void) {
  u8g2.drawCircle(32, 16, 14, U8G2_DRAW_ALL);
  u8g2.drawCircle(32, 16, 7, U8G2_DRAW_ALL);
  u8g2.drawLine(32, 0, 32, 32);
  u8g2.drawLine(15, 16, 49, 16);
  u8g2.drawFrame(0,0, 64,32);

}

void drawCrosshair(void) {
  u8g2.drawCircle(32, 16, 14, U8G2_DRAW_ALL);
  u8g2.drawCircle(32, 16, 7, U8G2_DRAW_ALL);
  u8g2.drawLine(32, 0, 32, 32);
  u8g2.drawLine(15, 16, 49, 16);
}

void drawEnvelope(void) {
  u8g2.drawFrame(52,0, 12,8);
  u8g2.drawLine(52,0, 58,4);
  u8g2.drawLine(58,4, 64,0);
}

void drawDot(void) {
  u8g2.drawEllipse(47, 4, 3, 3, U8G2_DRAW_ALL);
}

void drawAlarm(void) {
    u8g2.drawCircle(0, 0, 4, U8G2_DRAW_ALL); u8g2.sendBuffer();delay(300);
    u8g2.drawCircle(0, 0, 8, U8G2_DRAW_ALL); u8g2.sendBuffer();delay(300);
    u8g2.drawCircle(0, 0, 12, U8G2_DRAW_ALL); u8g2.sendBuffer();delay(800);
}

void drawClock(void) {
    u8g2.setFont(u8g2_font_unifont_t_symbols);
    u8g2.drawGlyph(0, 0, 0x23f0);
}


void drawText(void) {
   for (u8g2_uint_t i=10; i<=0; i--){
    u8g2.clearDisplay();u8g2.clearBuffer();
    //u8g2.setFont(u8g2_font_5x7_tr);
    //u8g2.setFont(u8g2_font_6x10_tf);
    u8g2.setFont(u8g2_font_ncenB08_tr);
    u8g2.setCursor(20, 32);
    //u8g2.println("Hello -ln");
    
    //u8g2.drawStr(0,40, "Hello -str");
    u8g2.println(packet);
    u8g2.print(i);
    u8g2.sendBuffer();
    delay(1000);
   }
}


void countDown10(void) {
   u8g2.setFont(u8g2_font_cu12_tr);	
   for (int i=10; i>=0; i--){
    u8g2.clearBuffer();
    u8g2.setCursor(35, 10);
    u8g2.print(i);
    u8g2.sendBuffer();
    delay(1000);
   }
}


void countdown1Min(void) {
  u8g2.setFont(u8g2_font_cu12_tr);	
  for (int i = 59; i >= 0; i--) {
    u8g2.clearBuffer();
    if (i >= 10) {
      u8g2.setCursor(35, 10);
      u8g2.print(i); 
    }
    if (i < 10) {
      u8g2.setCursor(35, 12);
      u8g2.print(i);
    }
    u8g2.sendBuffer();
    delay(50); // schneller gemacht statt 1000
  }
}

void makeDot(void) {
   u8g2.setFont(u8g2_font_cu12_tr);	
   u8g2.setCursor(0,0);
   u8g2.print(".");
   u8g2.sendBuffer();
   delay(500);
}
